
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

public class AnimationDisplay extends Panel implements Runnable {
    int framesPerSecond;
    long nextDisplayTime;
    long frameTimeDelta;
    long currentTimeMilliseconds;
    long targetTimeMilliseconds;
    Image backIm;
    Graphics bg;
    boolean drawing = true;
    int ww, wh;
    AdHocEnv env;
    Color bgColor;
    Thread animationThread;

    AnimationDisplay(int fps, AdHocEnv theEnv) {
	framesPerSecond = fps;
	frameTimeDelta = 1000L/framesPerSecond;
	env = theEnv;
    }

    void setBackgroundColor(Color bgC) {
	setBackground(bgC);
    }

    void startDrawing() {
	drawing = true;
	currentTimeMilliseconds = System.currentTimeMillis();
	targetTimeMilliseconds = currentTimeMilliseconds + frameTimeDelta;
	animationThread = new Thread(this);
	animationThread.start();
    }

    void stopDrawing() {
	drawing = false;
	animationThread = null;
    }


    public void paint(Graphics g) {
	update(g);
    }


  public void update(Graphics g) {
      while (getSize().width==0) ;
      ww = getSize().width;
      while (getSize().height==0) ;
      wh = getSize().height;
      if (backIm == null ||
	  backIm.getWidth(this) < ww ||
	  backIm.getHeight(this) < wh) {
	  backIm = createImage(ww,wh);
	  bg = backIm.getGraphics();
      }

      // clear window
      bg.clearRect(0,0,ww,wh);

      // draw each object
      env.draw(bg);

      // draw the accumulated background image
      g.drawImage(backIm, 0, 0, this);
  }


  public void run () {       // draw at regular intervals
      while (drawing) {
	currentTimeMilliseconds = System.currentTimeMillis();
	if (currentTimeMilliseconds < targetTimeMilliseconds) {
	    try {
		Thread.sleep(targetTimeMilliseconds - currentTimeMilliseconds);
	    }
	    catch (InterruptedException ie) {
		System.out.println("Interrupted!");
	    }
	}

	targetTimeMilliseconds = targetTimeMilliseconds + frameTimeDelta;
	repaint();
      }
  }
}





