package org.mitre.sim.pgm.intro;
import org.mitre.sim.api3.*;
import java.util.Random;
import java.util.Iterator;

public class HerdManagerModel extends Entity {
  private static int minimumPopulation = 3;
  private static int maximumPopulation = 5;
  private static double adultAge = 4.0;
  private long serial = 1;
  private int initialPopulationSize = 5;
  private double infantDeathAge = 4.0;
  private double adultDeathAge = 20.0;
  private double probabilityOfInfantDeath = 0.2;
  private double birthInterval = 7.0;
  private Random stream = new Random(1999);
  private Population elkHerd, adultsInHerd;

  public HerdManagerModel() { super("Manager"); }

  public void agenda() {
    elkHerd = createPopulation(ElkModel.class);
    adultsInHerd = elkHerd.applyFilter(new AdultFilter());
    for (int s = 0; s < initialPopulationSize; s++) register(createElk());
    Trigger herdTooSmall = elkHerd.sizeBelow(minimumPopulation);
    Trigger herdTooBig = adultsInHerd.sizeExceeds(maximumPopulation);
    schedule("eventNormalBirth", birthInterval);
    while(true) {
      WaitResult wr = waitForActionOrTrigger(herdTooSmall, herdTooBig);
      if (wr.triggerOccurred() && wr.trigger() == herdTooSmall) {
        replenishHerd(minimumPopulation);
      }
      else if (wr.triggerOccurred() && wr.trigger() == herdTooBig) {
        cullHerd(maximumPopulation / 2);
      }
    }
  }

  public void eventNormalBirth() {
    info("Normal birth");
    register(createElk());
    schedule("eventNormalBirth", birthInterval);
  }

  private void replenishHerd(int number) {
    info("Replenishing herd by " + number);
    for (int s = 0; s < number; s++)  register(createElk());
  }

  private void cullHerd(int number) {
    info("Culling herd by " + number);
    Iterator iter = adultsInHerd.iterator();
    for (int i = 0; i < number && iter.hasNext(); i++) {
      ((ElkModel)iter.next()).actionKill();
    }
  }

  public class AdultFilter implements Filter {
    public boolean passesFilter(Entity e) {
      return ((ElkModel)e).getAge() >= adultAge;
    }
  }

  private ElkModel createElk() {
    return new ElkModel("Elk-" + serial++,
                        (stream.nextDouble() <= probabilityOfInfantDeath)
                        ? infantDeathAge : adultDeathAge);
  }
}